#!/bin/sh
export PATH=.:/opt/bin:/opt/sbin/:/bin:/sbin:/usr/bin:/usr/sbin

echo "Entware installation started"

chmod +x ./wget

# check /opt already mounted
/bin/grep -q /opt /proc/mounts && echo "/opt already mounted" &&  exit 0

# make dir "opt" on the drive root
/bin/mkdir -p /media/DISK_A1/opt
[ ! -d "/media/DISK_A1/opt" ] && echo "cannot create /opt folder on hdd" && exit 0

# mount /opt (bind only)
mount -o bind "/media/DISK_A1/opt" /opt
if [ $? -ne 0 ] ; then
	echo "Mount /media/DISK_A1/opt to /opt FAILED! WTF?"
	exit 1
fi

echo Info: Checking for prerequisites and creating folders...
for folder in bin etc include lib sbin share tmp usr var home root
do
  if [ -d "/opt/$folder" ]
  then
    echo Warning: Folder /opt/$folder exists!
    echo Warning: If something goes wrong please clean /opt folder and try again.
  else
    mkdir /opt/$folder
  fi
done
[ -d "/opt/lib/opkg" ] || mkdir -p /opt/lib/opkg
[ -d "/opt/var/lock" ] || ln -s /var/lock /opt/var/lock
[ -d "/opt/var/log" ] || mkdir -p /opt/var/log
[ -d "/opt/var/run" ] || ln -s /var/run /opt/var/run
[ -d "/opt/etc/firewall.d" ] || mkdir -p /opt/etc/firewall.d
[ -d "/opt/lib/modules/2.6.23-rt" ] || mkdir -p /opt/lib/modules/2.6.23-rt
ln -s ./2.6.23-rt /opt/lib/modules/current


# full access for /opt/tmp & /var/lock may be needed in multiuser environment
chmod 777 "/opt/tmp"
chmod 777 "/opt/var/lock"

echo Info: Opkg package manager deployment...
wget http://entware.wl500g.info/binaries/entware/installer/opkg -O /opt/bin/opkg
chmod +x /opt/bin/opkg
wget http://entware.wl500g.info/binaries/entware/installer/opkg.conf -O /opt/etc/opkg.conf

echo Info: Basic packages installation...
/opt/bin/opkg update
/opt/bin/opkg install uclibc-opt
/opt/bin/opkg install dropbear
/opt/bin/opkg install terminfo
/opt/bin/opkg install wget
/opt/bin/opkg install busybox

echo "Generating rsa/dss keys for dropbear. It takes some time...."
rsa_key="/opt/etc/dropbear/dropbear_rsa_host_key"
dss_key="/opt/etc/dropbear/dropbear_dss_host_key"

rm -f "$rsa_key"
rm -f "$dss_key"
[ ! -d "/opt/etc/dropbear" ] && mkdir -p -m 755 /opt/etc/dropbear
/opt/bin/dropbearkey -t rsa -f "$rsa_key"
/opt/bin/dropbearkey -t dss -f "$dss_key"
chmod 600 "$rsa_key"
chmod 600 "$dss_key"
mkdir /opt/etc/dropbear/authorized_keys
chmod 600 /opt/etc/dropbear/authorized_keys

tar -xzf ./entware_keenetic_files.tar.gz -C /media/DISK_A1
# copy all but passwd shells group from keenetic to hdd
find /etc -mindepth 1 -maxdepth 1 ! -name passwd ! -name shells ! -name group -exec cp {} -RPf /media/DISK_A1/system/etc \;
# mount /etc (bind only)
mount -o bind "/media/DISK_A1/system/etc" /etc

SWAPFILE="/opt/.swapfile"
echo "Creating swap file. Please wait..."
dd if=/dev/zero of=$SWAPFILE bs=1048576 count=128
sync
mkswap $SWAPFILE 2> /dev/null



echo Info: Cleanup...
if [ -e "./entware_keenetic_install.sh" ]
then
  rm -f ./entware_keenetic_install.sh
  rm -f ./wget
  rm -f ./entware_keenetic_files.tar.gz
fi
echo "Starting Entware ...."
/opt/etc/init.d/rc.unslung start

echo "Info: Congratulations!"
echo "Info: If there are no errors above then Entware successfully initialized."
echo "Info: Found a Bug? Please report at forum.zyxmon.org & entware.wl500g.info"
